#include <iostream>
#include <stdlib.h>
#include <sstream>
#include <fstream>
#include <math.h>
using namespace std;

//General headers
#include "CommandLine.h"
#include "ErrorHandler.h"
#include "Miscellaneous.h"
#include "State.h"

//COLD-specific headers
#include "TreeLikelihood.h"
#include "MixTreeLikelihood.h"

//Headers for Output
#include "markupcommands.h"
#include "placeholders.h"
#include "modifiers.h"
#include "markupcommands2.h"
#include "placeholders2.h"
#include "placeholders3.h"
#include "modifiers2.h"

class intlistelt{
  int data;
  intlistelt *next;
  intlistelt *prev;
public:
  intlistelt(int i,intlistelt *ne=NULL){data=i;prev=ne;next=NULL;if(ne!=NULL){ne->next=this;};};
  friend class intlist;
  friend ostream& operator <<(ostream& out,intlistelt s);
};

ostream& operator <<(ostream& out,intlistelt s){
  out<<s.data<<"\n";
  if(s.next!=NULL){
    out<<*(s.next);
  };
  return out;
};

class intlist{//extensible list accessed serially.
  intlistelt *start;
  intlistelt *end;
  intlistelt *current;
public:
  intlist(){start=NULL;end=NULL;current=NULL;};
   int pop(){if(end==NULL){return -1;};if(current==end){current=NULL;};intlistelt *ne=end->prev;int ans=end->data;delete end;end=ne;if(end==NULL){start=NULL;};return ans;};
  void push(int i){intlistelt *x=new intlistelt(i,end);end=x;if(start==NULL){start=x;};};
  int get(){if(current==NULL){current=start;return -1;};int ans=current->data;current=current->next;current++;return ans;};
  void del(){if(current==NULL){return ;};if(current==start){start=current->next;};if(current==end){end=current->prev;};intlistelt *pr=current->prev;intlistelt *ne=current->next;delete current;if(pr!=NULL){pr->next=ne;};if(ne!=NULL){ne->prev=pr;};current=ne;};
  void reset(){current=start;};
  friend ostream& operator <<(ostream& out,intlist s);
};

ostream& operator <<(ostream& out,intlist s){
  if(s.start!=NULL){
    out<<*(s.start);
  }else{
    out<<"Empty list.\n";
  };
  return out;
};

class indexentry{
public:
  indexentry(const char *l,int j);
  char c;
  int i;
  indexentry *match;
  indexentry *next;
  indexentry *prev;
  void remove();
};

void indexentry::remove(){
  if(match!=NULL){
    match->remove();
    delete match;
  };
  if(prev!=NULL){
    prev->remove();
    delete prev;
  };
  if(next!=NULL){
    next->remove();
    delete next;
  };
};

indexentry::indexentry(const char *l,int j){
  c=*l;
  prev=NULL;
  next=NULL;
  if(*(l+1)=='\0'){
    match=NULL;
    i=j;
  }else{
    i=-1;
    match=new indexentry(l+1,j);
  };
};

class Index{
  indexentry *start;
public:
  Index(){start=NULL;};
  int lookup(istream &s);
  int lookup(const char *s);
  void insert(const char *ent,int i);
  void insert(istream& s);
  void insert(int num,const char *data[]);
  ~Index(){if(start!=NULL){start->remove();delete start;};};
};

void Index::insert(istream& s){
  while(s.good()&&!s.eof()){
    char* ent=readnonemptystring(s," \t\r\n");
    char* num=readnonemptystring(s,"\n");
    int i=atoi(num);
    this->insert(ent,i);
    delete[] ent;
    delete[] num;
  };
};

void Index::insert(int num,const char *data[]){
  for(int i=0;i<num;i++){
    int j=0;
    for(;*(data[i]+j)==' '&&*(data[i]+j)=='\t'&&*(data[i]+j)=='\r';j++);
    int k=j;
    for(;*(data[i]+j)!=' '&&*(data[i]+j)!='\t'&&*(data[i]+j)!='\r';j++);
    char *temp=new char[j-k+1];
    for(int l=0;l<j-k;l++){
      *(temp+l)=*(data[i]+k+l);
    };
    *(temp+j-k)='\0';
    int ii=atoi(data[i]+j);
    this->insert(temp,ii);
    delete[] temp;
  };
};

void Index::insert(const char *ent,int i){
  if(start==NULL){
    start=new indexentry(ent,i);
    return;
  };
  const char *l=ent;
  indexentry* current=start;
  for(;*l&&*l!=' '&&*l!='\t'&&*l!='\r'&&*l!='\n';){
    if(*l==current->c){
      l++;
      if(*l=='\0'||*l==' '||*l=='\t'||*l=='\r'||*l=='\n'){
	current->i=i;
	return;
      };
      if(current->match==NULL){
	current->match=new indexentry(l,i);
	return;
      }else{
	current=current->match;
      };
    }else if(*l<current->c){
      if(current->prev==NULL){
	current->prev=new indexentry(l,i);
	return;
      }else{
	current=current->prev;
      };
    }else{
      if(current->next==NULL){
	current->next=new indexentry(l,i);
	return;
      }else{
	current=current->next;
      };
    };
  };
};

int Index::lookup(istream &s){
  indexentry* current=start;
  for(char c=s.get();s.good()&&!s.eof()&&c!=' '&&c!='\t'&&c!='\r'&&c!='\n';){
    if(c==current->c){
      c=s.get();
      if(!s.good()||s.eof()||c==' '||c=='\t'||c=='\r'||c=='\n'){
	return current->i;
      };
      if(current->match==NULL){
	return -1;
      }else{
	current=current->match;
      };
    }else if(c<current->c){
      if(current->prev==NULL){
	return -1;
      }else{
	current=current->prev;
      };
    }else{
      if(current->next==NULL){
	return -1;
      }else{
	current=current->next;
      };
    };
  };
  return -1;
};

int Index::lookup(const char *s){
  indexentry* current=start;
  for(char c=*s;c&&c!=' '&&c!='\t'&&c!='\r'&&c!='\n';){
    if(c==current->c){
      c=*(++s);
      if(c=='\0'||c==' '||c=='\t'||c=='\r'||c=='\n'){
	return current->i;
      };
      if(current->match==NULL){
	return -1;
      }else{
	current=current->match;
      };
    }else if(c<current->c){
      if(current->prev==NULL){
	return -1;
      }else{
	current=current->prev;
      };
    }else{
      if(current->next==NULL){
	return -1;
      }else{
	current=current->next;
      };
    };
  };
  return -1;
};

class MarkupCommands{//A set of possible markup commands.
  Index commandnumbers;
public:
  MarkupCommands(const char *filename){ifstream in(filename);commandnumbers.insert(in);};
  MarkupCommands(int num=NUMMARKUPCOMMANDS,const char *data[]=markupcommands){commandnumbers.insert(num,data);};
  int find(istream& s);//checks whether the given option is a valid markup command
  int find(const char *s);
  char apply(int i,char c);
  void end(int i,ostream &out);
  void begin(int i,ostream &out);
};

int MarkupCommands::find(istream &s){
  return commandnumbers.lookup(s);
};

int MarkupCommands::find(const char *s){
  return commandnumbers.lookup(s);
};

#include "Markup.cpp"

class intcomp{//A computation to produce an integer
  const int *ans;
  const int *(*getval)();
public:
  intcomp(const int *(*gv)()){ans=NULL;getval=gv;};
  intcomp(const long double *(*gv)()){fatalError("Compilation error: Mismatched type in .placeholders file");};
  intcomp(void (*gv)(ostream& out)){fatalError("Compilation error: Mismatched type in .placeholders file");};
  int lookup(int i){if(ans==NULL){ans=(*getval)();};return *(ans+i);};
};

class ldcomp{//A computation to produce an integer
  const long double *ans;
  const long double *(*getval)();
public:
  ldcomp(const long double *(*gv)()){ans=NULL;getval=gv;};
  ldcomp(const int *(*gv)()){fatalError("Compilation error: Mismatched type in .placeholders file");};
  ldcomp(void (*gv)(ostream& out)){fatalError("Compilation error: Mismatched type in .placeholders file");};
  ldcomp(const long double *a){ans=a;getval=NULL;};
  long double lookup(int i){if(ans==NULL){ans=(*getval)();};return *(ans+i);};
};

class strcomp{//A computation to produce an integer
  void (*getval)(ostream&);
public:
  strcomp(void (*gv)(ostream&)){getval=gv;};
  strcomp(const int *(*gv)()){fatalError("Compilation error: Mismatched type in .placeholders file");};
  strcomp(const long double *(*gv)()){fatalError("Compilation error: Mismatched type in .placeholders file");};
  void lookup(ostream&out,int i){(*getval)(out);};
};

class PlaceHolders{//The possible variables to be included in the output
  void **computations;//These either compute the answer, or lookup a saved answer.
  Index names;
  Index mods;
  char *type;//Gives the type of each number.
  char *modparams;//Gives the type of parameters for each modifier.
  /*

    i   int
    e   enum
    d   long double
    s   string
    I   int list
    E   enum list
    D   long double list
    S   string list  

   */
public:
  PlaceHolders(const char *filename,const char *filename2=MAKESTRING(DEFAULTMODIFIERS)){ifstream in(filename);/* commandnumbers.insert(in);*/};
  PlaceHolders(int y=NUMPLACEHOLDERS,const char *phdata[]=placeholders,int z=NUMMODIFIERS,const char *mddata[]=modifiers);
  int find(istream& i){return names.lookup(i);};
  int find(char *&i){return names.lookup(i);};
  char gettype(int i){return *(type+i);};
  int getmod(istream &i){return mods.lookup(i);};
  int getmod(const char *i){return mods.lookup(i);};
  int getintval(int i,int sel=0);
  long double getdoubval(int i,int sel=0);
  void getstrval(int i,ostream&out, int sel=0);
};

int PlaceHolders::getintval(int i,int sel){
  if(*(type+i)!='i'&&*(type+i)!='I'&&*(type+i)!='e'&&*(type+i)!='E'){
    return -1;//Hopefully an error value.
  };
  return ((intcomp *)(computations+i))->lookup(sel);
};

long double PlaceHolders::getdoubval(int i,int sel){
  if(*(type+i)!='d'&&*(type+i)!='D'){
    return -1;//Hopefully an error value.
  };
  return ((ldcomp *)(computations+i))->lookup(sel);
};

void PlaceHolders::getstrval(int i,ostream&out,int sel){
  if(*(type+i)!='s'&&*(type+i)!='S'){
    return;
  };
  ((strcomp *)(computations+i))->lookup(out,sel);
};

#ifndef PLACEHOLDERINCLUDEFILE
#define PLACEHOLDERINCLUDEFILE "Placeholders.cpp"
#endif

#include PLACEHOLDERINCLUDEFILE

void assignfunctions(void** array,const char *type){
#define FUNCTIONNAMEIND2(a) PLACEHOLDER##a
#define FUNCTIONNAMEIND(a) FUNCTIONNAMEIND2(a)
#define FUNCTIONNAME(a) FUNCTIONNAMEIND(a)
#define NUMLOOPS NUMPLACEHOLDERS-1
#define LOOPINCLUDEFILE "OutputFunctionassign.cpp"
#include "preprocessorloop.cpp"
};

PlaceHolders::PlaceHolders(int y,const char *phdata[],int z,const char *mddata[]){
  type=new char[y];
  for(int i=0;i<y;i++){
    *(type+i)=*phdata[i];
    int j=1;
    for(;*(phdata[i]+j)==' '||*(phdata[i]+j)=='\t'||*(phdata[i]+j)=='\r'||*(phdata[i]+j)=='\n';j++);
    names.insert(phdata[i]+j,i);
  };
  modparams=new char[z];
  for(int i=0;i<z;i++){
    *(modparams+i)=*mddata[i];
    int j=1;
    for(;*(mddata[i]+j)==' '||*(mddata[i]+j)=='\t'||*(mddata[i]+j)=='\r'||*(mddata[i]+j)=='\n';j++);
    mods.insert(mddata[i]+j,i);
  };
  computations=new void*[NUMPLACEHOLDERS];
  assignfunctions(computations,type);
};

class OutputTemplate{
  const static char openmarkup='<';
  const static char closemarkup='>';
  const static char openplaceholder='{';
  const static char closeplaceholder='}';
  const static char separate='|';
  const static char endrow=';';
  const static char tableentrysep='&';
  const static char escape='\\';
  const static char* const specialesc;//="t";
  const static char* const specialescout;//="\t";
  MarkupCommands m;
  PlaceHolders p;
  istream& t;
  ostream& out;
  unsigned int level;//number of levels of conditional statements.
  //  unsigned int markup;//collection of flags indicating various markup formats.
  intlist mkup;//stack of markup commands applied. Used for calculating new markup flags when one is ended.
  intlist tr;//List of rows in table to be produced.
public:
  OutputTemplate(istream &tmp): t(tmp), out(cout){};
  OutputTemplate(const char *otf): t(*new ifstream(otf)), out(cout){};
  int readchar();//Reads a character in normal mode, and decides what to do with it. Returns 1 if read OK, and 0 if EOF or other error.
  void readmarkup();//After the openmarkup character, reads the name of the markup command, and deals appropriately with it.
  void readplaceholder();//Deals with a placeholder **********
  void stopmarkup();//kills the latest markup command
  void readmarkup(char *&pos);//After the openmarkup character, reads the name of the markup command, and deals appropriately with it.
  void readplaceholder(char *&pos,int num);//Deals with a placeholder **********
  char* tablerow();//collects a tablerow.
  char* tablerow(const char *outrow);//collects a tablerow.
  void tablerow(char* tr,int i);//processes the table row for case i
  void markup(char c);
  void closecondition();//reads to the end of the placeholder that started the condition.
  void closecondition(char *&pos);//reads to the end of the placeholder that started the condition.
  void processtablerow(char *trw);//Deals with the table row - First decides which instances of the row are necessary, then uses the function tablerow to create them
  void gotocase(int v);
  void gotocase(int v,char *&pos);
};

const char* const OutputTemplate::specialesc="t";
const char* const OutputTemplate::specialescout="\t";

void OutputTemplate::readmarkup(){
  int i=m.find(t);
  mkup.push(i);
  m.begin(i,out);
};

void OutputTemplate::readmarkup(char *&pos){
  int i=m.find(pos);
  for(;*pos&&*pos!=' '&&*pos!='\t'&&*pos!='\n'&&*pos!='\r';pos++);
  mkup.push(i);
  m.begin(i,out);
};


char *makestr(char c){
  static char ans[2];
  ans[1]='\0';
  ans[0]=c;
  return ans;
};

char *OutputTemplate::tablerow(){
  try{
    return readmatchedstring(t,openmarkup,closemarkup,escape,makestr(endrow));
  }catch(int i){
    if(i>0){
      fatalError("Output Template ended before table row complete.");
    }else{
      fatalError("Table row not ended before table completed.");
    };
  }catch(char c){
    if(c=='\\'){
      fatalError("Output Template ended during escape sequence.");
    }else{
      fatalError("Undefined error reading table row.");
    };
  };
};

char *OutputTemplate::tablerow(const char *outrow){
  try{
    stringstream s(outrow,ios_base::in);
    return readmatchedstring(s,openmarkup,closemarkup,escape,makestr(endrow));
  }catch(int i){
    if(i>0){
      fatalError("Output Template ended before table row complete.");
    }else{
      fatalError("Table row not ended before table completed.");
    };
  }catch(char c){
    if(c=='\\'){
      fatalError("Output Template ended during escape sequence.");
    }else{
      fatalError("Undefined error reading table row.");
    };
  };
};

void OutputTemplate::processtablerow(char *trw){
  //First form the list of all instances of this row


};

void OutputTemplate::tablerow(char* trw,int i){
  //processes the row for case i.
  for(;*trw;){
    char a=*(trw++);
    switch(a){
    case openmarkup:
      this->readmarkup(trw);
      break;
    case closemarkup:
      this->stopmarkup();
      break;
    case openplaceholder:
      this->readplaceholder(trw,i);
      break;
    case closeplaceholder:
      if(level==0){
	fatalError("Corrupted Output Template file - missing {");
      };
      level--;
      break;
    case endrow:
      //Should not happen
      break;
    case separate:
      if(level==0){
	this->markup(a);
      }else{
	this->closecondition(trw);
      };
      break;
    case tableentrysep:

      break;
    case escape:
      {
	a=*(trw++);
	int i=0;
	for(;*(specialesc+i)&&*(specialesc+i)!=a;i++);
	if(*(specialesc+i)){
	  this->markup(*(specialescout+i));
	  break;
	};
      };
    default:
      this->markup(a);
    };
  };
};


void OutputTemplate::gotocase(int v,char *&pos){
  int w=atoi(pos);
  for(;*pos&&*pos!=':';pos++);
  while(w!=v){
    char c=' ';
    for(;c&&c!='|'&&c!='}';c=*(pos++)){
      if(c=='{'){
	int lev=1;
	for(;lev>0;c=*(pos++)){
	  if(c=='{'){
	    lev++;
	  }else if(c=='}'){
	    lev--;
	  }else if(c=='\\'){
	    pos++;
	  };
	};
      }else if(c=='\\'){
	pos++;
      };
    };
    if(c=='|'){
      w=atoi(pos);
      level++;
      //skip past :
      for(;*pos&&*(pos++)!=':';);
    }else{
      w=v;
    };
  };//Add default cases, ranges, etc.
};

void OutputTemplate::gotocase(int v){
  int w;
  t>>w;
  while(w!=v){
    char c=' ';
    for(;c&&c!='|'&&c!='}';c=t.get()){
      if(c=='{'){
	int lev=1;
	for(;lev>0;c=t.get()){
	  if(c=='{'){
	    lev++;
	  }else if(c=='}'){
	    lev--;
	  }else if(c=='\\'){
	    t.get();
	  };
	};
      }else if(c=='\\'){
	t.get();
      };
    };
    if(c=='|'){
      t>>w;
      level++;
      //skip past :
      for(;!t.eof()&&t.get()!=':';);
    }else{
      w=v;
    };
  };//Add default cases, ranges, etc.
};

void OutputTemplate::readplaceholder(){
//Should probably rewrite this to use automatically generated code so
//that we can easily extend the available modifiers.
  int i=p.find(t);
  if(i==-1){//No match. Exit.
    this->closecondition();
    return;
  };
  char ty=p.gettype(i);
  switch(ty){
  case 'i':
  case 'e':
    {    
      int j=p.getmod(t);
      //      int prec=out.precision();
      switch(j){
      case MODIFIERS_CASES:
	{
	  int v=p.getintval(i);	
	  gotocase(v);
	  break;
	};
      default:
	out<<p.getintval(i);
      };
    };
    break;
  case 'd':
    //possible modifier "precision" - Note that the number may not be as accurate as the precision, so this may be meaningless.
    {    
      int j=p.getmod(t);
      int prec=out.precision();
      switch(j){
	case MODIFIERS_PRECISION:
	  t>>prec;
	};
      long double val=p.getdoubval(i);
      int oldprec=out.precision(prec);
      out<<val;
      out.precision(oldprec);
      break;
    };
  case 's':
    {    
      int j=p.getmod(t);
      int prec=out.precision();
      switch(j){
      case MODIFIERS_PRECISION:
	t>>prec;
      };
      int oldprec=out.precision(prec);
      p.getstrval(i,out);
      out.precision(oldprec);
      break;
    };
  case 'I':
  case 'E':
    {    
      int j=p.getmod(t);
      switch(j){
      case MODIFIERS_CASES:
	{
	  int v=p.getintval(i);	
	  int w;
	  t>>w;
	  for(;!t.eof()&&t.get()!=':';);
	  while(w!=v){
	    char c=' ';
	    for(;!t.eof()&&c!='|'&&c!='}';c=t.get()){
	      if(c=='{'){
		int lev=1;
		for(;lev>0;c=t.get()){
		  if(c=='{'){
		    lev++;
		  }else if(c=='}'){
		    lev--;
		  }else if(c=='\\'){
		    t.get();
		  };
		};
	      }else if(c=='\\'){
		t.get();
	      };
	    };
	    if(c=='|'){
	      t>>w;
	      //skip past :
	      for(;!t.eof()&&t.get()!=':';);
	    }else{
	      w=v;
	    };
	  };//Add default cases, ranges, etc.
	  break;
	};       
      default:
	out<<p.getintval(i);
      };
    };
    break;
  case 'D':
    {
      int j=p.getmod(t);
      int prec=out.precision();
      switch(j){
      case MODIFIERS_PRECISION:
	t>>prec;
      };
      int oldprec=out.precision(prec);
      p.getdoubval(i,0);//WRONG. NEED TO FIX THIS
      out.precision(oldprec);
      break;
    };
  case 'S':
    {    
      int j=p.getmod(t);
      int prec=out.precision();
      switch(j){
      case MODIFIERS_PRECISION:
	t>>prec;
      };
      int oldprec=out.precision(prec);
      p.getstrval(i,out);
      out.precision(oldprec);
      break;
    };
  };
};

void OutputTemplate::readplaceholder(char *&pos,int num){
//Should probably rewrite this to use automatically generated code so
//that we can easily extend the available modifiers.
  int i=p.find(pos);
  for(;*pos&&*pos!=' ';pos++);
  if(i==-1){//No match. Exit.
    this->closecondition(pos);
    return;
  };
  char ty=p.gettype(i);
  switch(ty){
  case 'i':
  case 'e':
    {    
      int j=p.getmod(pos);
      for(;*pos&&*pos!=' '&&*pos!='\n'&&*pos!='\t'&&*pos!='\r';pos++);
      for(;*pos==' '&&*pos=='\n'&&*pos=='\t'&&*pos=='\r';pos++);
      //      int prec=out.precision();
      switch(j){
      case MODIFIERS_CASES:
	{
	int v=p.getintval(i);	
	gotocase(v,pos);
	break;
	};
      default:
	out<<p.getintval(i);
      };
    };
    break;
  case 'd':
    //possible modifier "precision" - Note that the number may not be as accurate as the precision, so this may be meaningless.
    {    
      int j=p.getmod(pos);
      for(;*pos&&*pos!=' '&&*pos!='\n'&&*pos!='\t'&&*pos!='\r';pos++);
      for(;*pos==' '&&*pos=='\n'&&*pos=='\t'&&*pos=='\r';pos++);
      int prec=out.precision();
      switch(j){
	case MODIFIERS_PRECISION:
	  prec=atoi(pos);
	  if(*pos=='-'){
	    pos++;
	  };
	  for(;*pos>='0'&&*pos<='9';pos++);
	};
      long double val=p.getdoubval(i);
      int oldprec=out.precision(prec);
      out<<val;
      out.precision(oldprec);
      break;
    };
  case 's':
    {    //Need to add support for cases
      int j=p.getmod(pos);
      for(;*pos&&*pos!=' '&&*pos!='\n'&&*pos!='\t'&&*pos!='\r';pos++);
      for(;*pos==' '&&*pos=='\n'&&*pos=='\t'&&*pos=='\r';pos++);
      int prec=out.precision();
      switch(j){
      case MODIFIERS_PRECISION:
	prec=atoi(pos);
	if(*pos=='-'){
	  pos++;
	};
	for(;*pos>='0'&&*pos<='9';pos++);
      };
      int oldprec=out.precision(prec);
      p.getstrval(i,out);
      out.precision(oldprec);
      break;
    };
  case 'I':
  case 'E':
    {    
      int j=p.getmod(pos);
      for(;*pos&&*pos!=' '&&*pos!='\n'&&*pos!='\t'&&*pos!='\r';pos++);
      for(;*pos==' '&&*pos=='\n'&&*pos=='\t'&&*pos=='\r';pos++);
      switch(j){
      case MODIFIERS_CASES:
	{
	  int v=p.getintval(i,num);	
	  gotocase(v,pos);
	  break;
	};       
      default:
	out<<p.getintval(i,num);
      };
    };
    break;
  case 'D':
    {
      int j=p.getmod(pos);
      for(;*pos&&*pos!=' '&&*pos!='\n'&&*pos!='\t'&&*pos!='\r';pos++);
      for(;*pos==' '&&*pos=='\n'&&*pos=='\t'&&*pos=='\r';pos++);
      int prec=out.precision();
      switch(j){
      case MODIFIERS_PRECISION:
	prec=atoi(pos);
	if(*pos=='-'){
	  pos++;
	};
	for(;*pos>='0'&&*pos<='9';pos++);
      };
      int oldprec=out.precision(prec);
      p.getdoubval(i,num);//WRONG. NEED TO FIX THIS
      out.precision(oldprec);
      break;
    };
  case 'S':
    {    
      int j=p.getmod(pos);
      for(;*pos&&*pos!=' '&&*pos!='\n'&&*pos!='\t'&&*pos!='\r';pos++);
      for(;*pos==' '&&*pos=='\n'&&*pos=='\t'&&*pos=='\r';pos++);
      int prec=out.precision();
      switch(j){
      case MODIFIERS_PRECISION:
	prec=atoi(pos);
	if(*pos=='-'){
	  pos++;
	};
	for(;*pos>='0'&&*pos<='9';pos++);
      };
      int oldprec=out.precision(prec);
      p.getstrval(i,out,num);
      out.precision(oldprec);
      break;
    };
  };
};

void OutputTemplate::stopmarkup(){
  int i=mkup.pop();
  m.end(i,out);
};

void OutputTemplate::closecondition(){//reads to the end of the placeholder that started the condition.
  unsigned int l=0;
  for(char a=t.get();!t.eof();a=t.get()){
    switch(a){
    case escape:
      t.get();
      break;
    case openplaceholder:
      l++;
      break;
    case closeplaceholder:
      if(l==0){
	level--;
	return;
      };
      l--;
      break;
    };
  };
  //Only get here if condition is badly formated.
  fatalError("Corrupted Output Template file - missing }");
};

void OutputTemplate::closecondition(char *&pos){//reads to the end of the placeholder that started the condition.
  unsigned int l=0;
  for(char a=*(pos);a;a=*(++pos)){
    switch(a){
    case escape:
      pos++;
      break;
    case openplaceholder:
      l++;
      break;
    case closeplaceholder:
      if(l==0){
	level--;
	return;
      };
      l--;
      break;
    };
  };
  //Only get here if condition is badly formated.
  fatalError("Corrupted Output Template file - missing }");
};


void OutputTemplate::markup(char c){//Output the character c with any markup commands that may be appropriate.
  mkup.reset();
  for(int i=mkup.get();i!=-1;i=mkup.get()){
    c=m.apply(i,c);//Not right - apply can give multiple characters.
  };
  out<<c;
};

int OutputTemplate::readchar(){
  char a=t.get();
  if(t.eof()){
    return 0;
  };
  switch(a){
  case openmarkup:
    this->readmarkup();
    break;
  case closemarkup:
    this->stopmarkup();
    break;
  case openplaceholder:
    this->readplaceholder();
    break;
  case closeplaceholder:
    if(level==0){
      fatalError("Corrupted Output Template file - missing {");
    };
    level--;
    break;
  case endrow:
    //????
    break;
  case separate:
    if(level==0){
      this->markup(a);
    }else{
      this->closecondition();
    };
  case escape:
    {
      a=t.get();
      int i=0;
      for(;*(specialesc+i)&&*(specialesc+i)!=a;i++);
      if(*(specialesc+i)){
	this->markup(*(specialescout+i));
	break;
      };
    };
  default:
    this->markup(a);
  };
  return 1;
};


void outputtemplate(const OutputTemplate &tmplt,treelk *ff,long double *x,int *valpars=NULL){
  ostream *out=&cout;
  variable *rec=vars.seektag("-output");
  if(rec!=NULL){
    out=new ofstream(*(char **)rec->value);
    if(!((ofstream *)out)->is_open()){
      cout<<"Could not open file "<<*(char **)rec->value<<"\n";
      out=&cout;
    };
  };
  int num=1;
  rec=vars.seektag("-mixture");
  if(rec!=NULL){
    num=atoi(*(char **)rec->value);
  };
  calculationdata::setup(num,ff,x,valpars,1,valpars);
};


void outputnoconverge(int numtimes,treelk *ff,long double *x,int argc,char* argv[],int *valpars=NULL){
  ostream *out=&cout;
  variable *rec=vars.seektag("-output");
  if(rec!=NULL){
    out=new ofstream(*(char **)rec->value);
    if(!((ofstream *)out)->is_open()){
      cout<<"Could not open file "<<*(char **)rec->value<<"\n";
      out=&cout;
    };
  };
  for(int i=0;i<argc;i++){
    *(out)<<argv[i]<<" ";
  };
  *(out)<<"\n\n";
  if(valpars!=NULL){
    *(out)<<"Parameters selected:\n";
    for(int i=0;*(valpars+i)>0;i++){
      *(out)<<*(valpars+i)<<"  ";
    };
    *(out)<<"\n\n";
  };
  *(out)<<"Failed to converge after "<<numtimes<<" steps. Currently at:\n\n";
  out->precision(10);
  long double val=ff->quickEvaluate(x);
  ff->normalise();
  ff->printall(*out);
  *(out)<<"\n\n Log likelihood="<<val<<"\n\n";
  delete[] x;
  delete ff;
  if(rec!=NULL){
    ((ofstream *)out)->close();
    delete out;
  };
  exit(0);
};

void printconverge(treelk *ff,long double *x,int *valpars=NULL){
  if(valpars!=NULL){
    cout<<"Parameters selected:\n";
    for(int i=0;*(valpars+i)>0;i++){
      cout<<*(valpars+i)<<"  ";
    };
    cout<<"\n\n";
  };
  cout<<"Converged to:\n\n";
  cout.precision(10);
  ff->normalise();
  ff->printall(cout,valpars);
  ff->getx(x);
  long double val=ff->quickEvaluate(x);
  cout<<"\n\n Log Likelihood="<<val<<"\n\n";
  if(vars.seektag("-printsitelikes")){
    long double *sl=ff->sitelikes();
    cout<<"Site\t\tLikelihood\n\n";
    int sites=ff->numsites();
    for(int i=0;i<sites;i++){
      cout<<i+1<<"\t\t"<<*(sl+i)<<"\n";
    };
    delete[] sl;
  };
  if(vars.seektag("-tstats")){
    ff->tstat(x,cout);
  };
};

void outputconverge(treelk *ff,long double *x,int argc,char* argv[],int *valpars){
  ostream *out=&cout;
  variable *rec=vars.seektag("-output");
  if(rec!=NULL){
    out=new ofstream(*(char **)rec->value);
    if(!((ofstream *)out)->is_open()){
      cout<<"Could not open file "<<*(char **)rec->value<<"\n";
      out=&cout;
    };
  };
  (*out)<<CurrentState->getcommand()<<"\n\n";
  for(int i=0;i<argc;i++){
    *(out)<<argv[i]<<" ";
  };
  *(out)<<"\n\n";
  int num=1;
  rec=vars.seektag("-mixture");
  if(rec!=NULL){
    num=atoi(*(char **)rec->value);
  };
  *(out)<<"Converged to:\n\n";
  out->precision(10);
  long double val=ff->quickEvaluate(x);
  ff->normalise();
  ff->getx(x);
  Realmatrix oi=ff->oi(x);//Only really necessary if outputing
			  //something that depends on it.
  val=ff->evaluate(x);//already calculated this when calculating oi.
  ff->printtree(*out);
  (*out)<<"\n\n";
  if(vars.seektag("-blstdev")){
    (*out)<<"Standard deviation of branch lengths:\n\n";
    ff->printtreedev(oi,*out);    
    (*out)<<"\n\n";
  };  
  if(valpars!=NULL){
    (*out)<<"Parameters selected:\n";
    for(int i=0;*(valpars+i)>0;i++){
      (*out)<<*(valpars+i)<<"  ";
    };
    (*out)<<"\n\n";
  };
  long double *tc=ff->truepars();
  if(params::numpars>0){
    (*out)<<"Parameter";
    out->width(25);
    (*out)<<"Coefficient";
    out->width(40);
    (*out)<<"Exponential of Coefficient";
    if(vars.seektag("-tstats")){
      out->width(25);
      (*out)<<"t-statistic";
    };
    (*out)<<"\n\n";
    for(int nn=0;nn<num;nn++){
      for(int j=0;j<params::numpars;j++){
	out->width(5);
	if(valpars!=NULL&&j>0){
	  (*out)<<*(valpars+j-1);
	}else{
	  (*out)<<j;
	};
	out->width(29);
	(*out)<<*(tc+nn*params::numpars+j);
	out->width(30);
	(*out)<<exp(*(tc+nn*params::numpars+j));
	if(vars.seektag("-tstats")){
	  out->width(25);
	  (*out)<<*(tc+j)/sqrt(*(oi.entries+(oi.sz-params::numpars+j)*(oi.sz+1)));
	};
	(*out)<<"\n";
      };
      (*out)<<"\n\n";
    };
  };
  (*out)<<"\n\n Log Likelihood="<<val<<"\n\n";
  if(vars.seektag("-observedinformation")){
    (*out)<<"Observed information matrix:\n\n"<<oi<<"\n\n";
  };
  if(vars.seektag("-printsitelikes")){
    long double *sl=ff->sitelikes();
    *(out)<<"Site\t\tLikelihood\n\n";
    int sites=ff->numsites();
    for(int i=0;i<sites;i++){
      *(out)<<i+1<<"\t\t"<<*(sl+i)<<"\n";
    };
    delete[] sl;
  };
  delete[] tc;
  delete[] x;
  ff->printall(*out);
  if(vars.seektag("-posteriorprobs")!=NULL &&num>1){
    long double *ppr=((Mixtreelikelihood *)ff)->sitepostprobs();
    (*out)<<"Posterior probabilities:\n\n";
    (*out)<<"site number    posterior probabilities\n";
    for(int i=0;i<ff->numsites();i++){
      (*out)<<i+1<<"\t\t";
      for(int j=0;j<num;j++){
	(*out)<<*(ppr+i*num+j)<<"\t";
      };
      (*out)<<"\n";
    };
    (*out)<<"\n";
  };
  rec=vars.seektag("-likelihooddisplacement");
  if(rec!=NULL){
    int *sel=selection(*(const char *const *)rec->value);
    long double *ld=ff->likelihoodDisplacement(sel);
    (*out)<<"first derivatives:\n\n";
    int nv=0;
    for(;*(sel+nv)>=0;nv++);
    for(int i=0;i<ff->numsites();i++){
      for(int j=0;j<nv;j++){
	(*out)<<*(ld+ff->numsites()*ff->numsites()+i*nv+j)<<"  ";
      };
      (*out)<<"\n";
    };    
    Realmatrix R(ff->numsites(),ld);
    (*out)<<"\nLikelihood displacement:\n\n"<<R<<"\n\n";
    long double *x=new long double[ff->numsites()];
    Realmatrix A=R.diagonalisesymmetric(x);
    (*out)<<A<<"\n";
    for(int i=0;i<ff->numsites();i++){
      /*      for(int j=0;j<ff->numsites();j++){
	*(out)<<*(ld+i*ff->numsites()+j)<<"  ";
	};*/
      *(out)<<*(x+i)<<"\n";
    };      
    delete[] ld;
    delete[] x;
  };
  delete ff;
  if(vars.seektag("-output")!=NULL){
    ((ofstream *)out)->close();
    delete out;
  };
  if(vars.seektag("-keepstate")==NULL){
    rec=vars.seektag("-state");
    if(rec!=NULL){
      remove(*(char **)rec->value);
    }else{
      remove("statefile");
    };
  };
  exit(0);
};



